#include "Collision.h"
#include "GFX.h" // for debugdraw only
#include "CollisionGame.h" // DEBUG
#include <Math.h>


void CollisionManager::UpdateCollision() {
	collisionchecks = 0;
	collisions.clear();
	SAPBroadPhase(0);
	for(Collisions::iterator citer = collisions.begin(); citer != collisions.end(); citer++) {
		Collision col = (*citer);

		// TODO: Fix trigger stuff, modify translation vector for velocity?
		//if(!col.c1->isTrigger && !col.c2->isTrigger) {
			if(col.c1->isStatic) {
				col.c2->MoveTo(col.c2->center - col.translationVector);
				col.c2->CollidedWith(col.c1);
			}
			else if(col.c2->isStatic) {
				col.c1->MoveTo(col.c1->center + col.translationVector);
				col.c1->CollidedWith(col.c2);
			}
			else {
				col.c1->MoveTo(col.c1->center + col.translationVector/2.0);
				col.c2->MoveTo(col.c2->center - col.translationVector/2.0);
				col.c1->CollidedWith(col.c2);
				col.c2->CollidedWith(col.c1);
			}
		//}

	}
	//for(Colliders::iterator citer = colliders.begin(); citer != colliders.end(); citer++) {
	//	if((*citer)->isStatic) continue;
	//	BruteForceBroadPhase((*citer));
	//}
}

// Insert sorted... maybe we can bin sort this? Right now O(n) for inserts
void CollisionManager::AddCollider(Collider *col) {
	Colliders::iterator pos = colliders.begin();
	while(!colliders.empty() && pos != colliders.end() && (*pos)->center.x + (*pos)->aabb.p1.x < col->center.x + col->aabb.p1.x) {
		pos++;
	}
	colliders.insert(pos, col);
}

void CollisionManager::DebugDraw() {
	GFX *g = GFX::Instance();
	for(Colliders::iterator citer = colliders.begin(); citer != colliders.end(); citer++) {
		PolyCollider *col = (PolyCollider *)(*citer);
		for(int i = 0; i < col->numVerts; i++) {
			//g->GFXrectfill(col->verts[i].x-1, col->verts[i].y-1, col->verts[i].x+1, col->verts[i].y+1, Color(0, 1, 0));
			g->GFXline(col->verts[i].x, col->verts[i].y, col->verts[i].x + col->edges[i].x, col->verts[i].y + col->edges[i].y, Color(0, 150, 0));
		}
		//g->GFXrectfill(col->center.x + col->aabb.p1.x, col->center.y + col->aabb.p1.y, col->center.x + col->aabb.p2.x, col->center.y + col->aabb.p2.y, Color(0,1,0,0.3f));
	}
	for(Collisions::iterator citer = collisions.begin(); citer != collisions.end(); citer++) {
		Collision col = (*citer);
		//PolyCollider *colt = (PolyCollider *)col.c1;
		//GFX::Instance()->GFXrectfill(colt->center.x + colt->aabb.p1.x, colt->center.y + colt->aabb.p1.y, colt->center.x + colt->aabb.p2.x, colt->center.y + colt->aabb.p2.y, Color(1,0,0,0.3f));
		//colt = (PolyCollider *)col.c2;
		//GFX::Instance()->GFXrectfill(colt->center.x + colt->aabb.p1.x, colt->center.y + colt->aabb.p1.y, colt->center.x + colt->aabb.p2.x, colt->center.y + colt->aabb.p2.y, Color(1,0,0,0.3f));

		Vec2 p1 = col.c1->center;
		Vec2 p2 = col.c2->center;
		//if(p1.x < -100 || p1.y < -100 || p1.y < -100 || p2.y < -100) {
		//	g->GFXrectfill(0, 0, 500, 500, Color(5,5,5,0.5));
		//}
		g->GFXline(p1.x, p1.y, p2.x, p2.y, Color(200, 0, 0));
	}
	gfx->GFXDebug("Collisions: ", collisions.size(), SC_WIDTH/2, SC_HEIGHT - 80);
	gfx->GFXDebug("Collision Checks: ", collisionchecks, SC_WIDTH/2, SC_HEIGHT - 60);
}

void CollisionManager::SAPSortColliders() {
	Colliders::iterator i;
	Colliders::iterator j, jleft;

	i = colliders.begin();
	i++;
	for (; i != colliders.end(); i++)
	{
		Collider *curr = (*i);
		j = i;
		jleft = j;
		jleft--;

		while (j != colliders.begin() && (*jleft)->WorldMin().x > curr->WorldMin().x)
		{
			(*j) = (*jleft);
			j--;
			if(jleft != colliders.begin())
				jleft--;
		}

		(*j) = curr;
	}

	 // Test Integrity of sort
	/*float last = 0;
	for(i = colliders.begin(); i != colliders.end(); i++) {
		if(i == colliders.begin()) { last = (*i)->WorldMin().x; continue; }
		if((*i)->WorldMin().x < last) throw "Error: Sort Invalid!";
		last = (*i)->WorldMin().x;
	}*/
}

void CollisionManager::SAPBroadPhase(Collider *col) {
	Colliders::iterator citer;
	//Game::Instance()->test1 = false;
	//SAPSortColliders();	
	SAPSortColliders();

	for(citer = colliders.begin(); citer != colliders.end(); citer++) {
		Collider *curr = (*citer);
		Colliders::iterator other = citer;
		//if(curr->isStatic && (*other)->isStatic) continue;
		other++;
		while(other != colliders.end() && (*other)->WorldMin().x < curr->WorldMax().x) {

			Collision r = CheckCollision((PolyCollider *) curr, (PolyCollider *)(*other));
			//if(!r.collides) r = CheckCollision((PolyCollider *)(*other), (PolyCollider *) curr);
			if(r.collides) {
				PolyCollider *pc = (PolyCollider *) curr;
				//pc->MoveTo(pc->center + r.translationVector);
					
				//Game::Instance()->test1 = true;
				collisions.push_back(r);
			}
			
			other++;
		}
	}
}

void CollisionManager::BruteForceBroadPhase(Collider *col) {
	Colliders::iterator citer;
	//Game::Instance()->test1 = false;

	for(citer = colliders.begin(); citer != colliders.end(); citer++) {
		if((*citer) == col) continue;
		PolyCollider *pc = (PolyCollider *) col;
		Collision r = CheckCollision(pc, (PolyCollider *)(*citer));
		if(r.collides) {
			//pc->MoveTo(pc->center + r.translationVector);
			//Game::Instance()->test1 = true;
			collisions.push_back(r);
		}
	}
}

// SAT collision helper function adapted from code by Laurent-Cozic, licensed under the MIT license.
void CollisionManager::ProjectPolygon(Vec2 axis, PolyCollider *polygon, Vec2 *minMax) {
	float dotProduct = axis.dotProduct(polygon->verts[0]);
	minMax->x = dotProduct;
	minMax->y = dotProduct;
	for(int i = 0; i < polygon->numVerts; i++) {
		dotProduct = polygon->verts[i].dotProduct(axis);
		if(dotProduct < minMax->x){
			minMax->x = dotProduct;
		}
		else if(dotProduct > minMax->y) {
			minMax->y = dotProduct;
		}
	}
}

// SAT collision adapted from code by Laurent-Cozic, licensed under the MIT license.
Collision CollisionManager::CheckCollision(PolyCollider *c1, PolyCollider *c2) {
	int count1 = c1->numVerts;
	int count2 = c2->numVerts;
	Collision result;
	result.c1 = c1;
	result.c2 = c2;
	result.collides = true;
	result.willCollide = true;
	float minIntervalDist = 9999999;
	Vec2 edge;
	Vec2 translationAxis;

	collisionchecks++;
	for(int index = 0; index < count1 + count2; index++) {
		if(index < count1) {
			edge = c1->edges[index];
		}
		else {
			edge = c2->edges[index - count1];
		}

		Vec2 axis = Vec2(-edge.y, edge.x);
		axis = axis.Normalize();

		Vec2 minMax1, minMax2;
		ProjectPolygon(axis, c1, &minMax1);
		ProjectPolygon(axis, c2, &minMax2);

		float intervalDist = IntervalDistance(minMax1, minMax2);

		
		if(intervalDist > 0) {
			result.collides = false;
		}

		float velocityProjection = axis.dotProduct(c1->velocity - c2->velocity);

		if(velocityProjection < 0) {
			minMax1.x += velocityProjection;
		}
		else {
			minMax1.y += velocityProjection;
		}

		intervalDist = IntervalDistance(minMax1, minMax2);
		if(intervalDist > 0) result.willCollide = false;

		if(!result.collides && !result.willCollide) break;

		intervalDist = abs(intervalDist);
		if(intervalDist < minIntervalDist) {
			minIntervalDist = intervalDist;
			translationAxis = axis;

			Vec2 d = c1->center - c2->center;
			if(d.dotProduct(translationAxis) < 0) {
				translationAxis = Vec2(-translationAxis.x, -translationAxis.y);
			}
		}
	}

	if(result.willCollide) {
		result.translationVector = translationAxis * minIntervalDist;
	}

	return result;
}

float CollisionManager::IntervalDistance(Vec2 minMax1, Vec2 minMax2) {
	if(minMax1.x < minMax2.x) {
		return minMax2.x - minMax1.y;
	}
	else {
		return minMax1.x - minMax2.y;
	}
}