#pragma once
#include "Vec2.h"
#include "Enums.h"
#include "GFX.h"
#include "Collision.h"
#include <list>

class Renderer {
public:
	Vec2 pos;
	Color col;

	Renderer() { };
	virtual ~Renderer(){ };
	virtual void Render() = 0;
	virtual void Update(float dt) = 0;
	virtual void Destroy() = 0;
};

typedef std::list<Renderer *> Renderers;

class RenderManager {
public:
	void RemoveRenderer(Renderer *renderer, bool safe = false);
	void AddRenderer(Renderer *renderer);
	void Update(float dt);
	void Render();
protected:
	void CleanUp();
private:
	Renderers objects;
	Renderers deletionQueue;
};

class Timer {
public:
	Timer();
	void Start();
	void Stop();
	void Pause();
	void Unpause();
	int GetTicks();
	bool isPaused();
	bool isStarted();

private:
	bool started;
	bool paused;
	int startTicks;
	int pausedTicks;
};

class Core : public RenderManager, public CollisionManager {
public:
	static Core *Instance();

	Core *core;
	GFX *gfx;

	bool Start();
	bool Init();
	float DeltaTime(){ return deltaTime; }

	bool InCameraRange(float x1, float y1, float x2, float y2);

	void SetCamera(Vec2 newpos);
	Renderer *tracking;
	float cameraZoom;
	Vec2 cameraCenter;
	void TogglePause();

	Vec2 GetUpperClip();
	Vec2 GetLowerClip();

	int screenHeight;
	int screenWidth;

protected:
	Core();
	bool init_GL();
private:
	// Make a separate class for camera?
	float deltaTime;
	bool paused;
	// Singleton instance
	static Core *instance;
};

extern GFX *gfx;