/* GFX.cpp
	Designed as a wrapper for graphics libraries. Using defines, we should be able to alter
	which graphics library we use to increase portability. Right now only allegro is supported.
	It is also a singleton, and is used as the single entry point for drawing to the screen.
*/

#include "Core.h"
#include "GFX.h"
#include <sstream>
#define BUILD_SDL
#ifdef BUILD_SDL
#include "SDL.h"
#include "SDL_opengl.h"
#include "Text.h"

#endif


//weren
GFX* GFX::instance = 0;

GFX* GFX::Instance() {
	if(instance == 0) {
		instance = new GFX();
	}
	return instance;
}

GFX::GFX() {
	c = Core::Instance();
	init();
}

#ifdef BUILD_SDL
void GFX::GFXupdate() {

	GFXDebug("Draw Calls: ", drawcalls, 0, 0);
	GFXDebug("Culled: ", culled, 0, 0);
	float ypos = 750;
	float czoom = Core::Instance()->cameraZoom;
	Vec2 ccenter = Core::Instance()->cameraCenter;
	glColor4f(1,1,1,1);

	for(std::list<string>::iterator i = debugDisplay.begin(); i != debugDisplay.end(); i++) {
		glPushMatrix();
		// Remove Camera offset (Center is negative)
		glTranslatef((ccenter.x), ((ccenter.y )), 1);
		
		// Remove Camera Zoom
		glScalef(1.0f / czoom, 1.0f /czoom, 1);
		glTranslatef(0, ypos, 1);
		glScalef(1, -1, 0);
		if(debugFont)
		debugFont->Render((*i).c_str());
		glPopMatrix();
		ypos -= 20;
	}
	debugDisplay.clear();
	SDL_GL_SwapBuffers();
	//SDL_Flip( screen );
}

GFX::~GFX() {
	//destroy_bitmap(GFXbuffer);
}

void GFX::GFXrectfill(float x1, float y1, float x2, float y2, Color col) {
	if(!c->InCameraRange(x1, y1, x2, y2)) {
		culled++;
		return;
	}
	drawcalls++;
	    //Move to offset
		glPushMatrix();
    glTranslatef( x1, y1, 0 );

    //Start quad
    glBegin( GL_QUADS );

        glColor4f( col.r, col.g, col.b, col.a);

        //Draw square
	    glVertex3f( 0,			0,			0 );
	    glVertex3f( x2 - x1,	0,			0 );
	    glVertex3f( x2 - x1,	y2 - y1,	0 );
	    glVertex3f( 0,			y2 - y1,	0 );

    //End quad
    glEnd();
	glPopMatrix();
    //Reset
 //   glLoadIdentity();

	//rectfill(GFXbuffer, x1, y1, x2, y2, makecol(col.r, col.g, col.b));
}

void GFX::GFXtriangle(float x1, float y1, float x2, float y2, float x3, float y3, Color col) {
	drawcalls++;
	//triangle(GFXbuffer, x1, y1, x2, y2, x3, y3, makecol(col.r, col.g, col.b));
}

void GFX::GFXline(float x1, float y1, float x2, float y2, Color col) {
	if(!c->InCameraRange(x1, y1, x2, y2)) {
		culled++;
		return;
	}
		
	drawcalls++;
	glPushMatrix();
	glTranslatef( x1, y1, 0 );
	glBegin(GL_LINES);
	    glColor4f( col.r, col.g, col.b, col.a);
		glVertex3f(0.0f, 0.0f, 0.0f); // origin of the line
		glVertex3f(x2 - x1, y2 - y1, 0.0f); // ending point of the line
	glEnd( );
	glPopMatrix();
	//glLoadIdentity();
	//fastline(GFXbuffer, x1, y1, x2, y2, makecol(col.r, col.g, col.b));
}

void GFX::GFXclearTo(Color col) {
	drawcalls = 0;
	culled = 0;
	//clear_to_color(GFXbuffer, makecol(col.r, col.g, col.b));
}

void GFX::GFXputpixel(float x, float y, Color col) {
	drawcalls++;
	//putpixel(GFXbuffer, x, y, makecol(col.r, col.g, col.b));
}

bool GFX::init() {
	drawcalls = 0;
	culled = 0;
	debugFont = new FTTextureFont("morgue.ttf");

	if(debugFont->Error())
	{
		fprintf(stderr, " Could not load debug font!\n");
		return false;
	}
	debugFont->FaceSize(20);
	//font->FaceSize(size);
	debugFont->Depth(10);
	debugFont->Outset(0, 3);
	debugFont->CharMap(ft_encoding_unicode);

	return true;
}



void GFX::GFXDebug(char *stre, double val, float x, float y) {
	std::stringstream dc;
	dc << val;
	std::string str = stre + dc.str();
	char *stringer = (char *)str.c_str();
	debugDisplay.push_back(std::string(stringer));
}

void GFX::GFXDebug(char *stre, Vec2 val) {
	std::stringstream dc;
	dc << "(" << val.x << ", " << val.y << ")";
	std::string str = stre + dc.str();
	char *stringer = (char *)str.c_str();
	debugDisplay.push_back(std::string(stringer));
}

void GFX::GFXtext(char *str, float x, float y, Color col, Color mask) {
	drawcalls++;

}

void GFX::GFXtext_center(char *str, float x, float y, Color col, Color mask) {
	drawcalls++;

}

#endif